/*
 * Mirage - High Performance Music Similarity and Automatic Playlist Generator
 * http://hop.at/mirage
 * 
 * Copyright (C) 2007 Dominik Schnitzer <dominik@schnitzer.at>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */


using System;
using System.Resources;
using System.IO;
using System.Reflection;

namespace Mirage
{
	
public class Mfcc
{

	Matrix filterWeights;
	Matrix dct;
	
	public Mfcc(int winsize, int srate, int filters, int cc)
	{
		Assembly assem = this.GetType().Assembly;
		
		// Load the DCT
		dct =
			Matrix.Load(assem.GetManifestResourceStream("dct.raw"));
			
		// Load the MFCC filters from the filter File.
		filterWeights =
			Matrix.Load(assem.GetManifestResourceStream("filterweights.raw"));
	}
	
	
	public Matrix Apply(Matrix m)
	{

		Timer t = new Timer();
		t.Start();

		Matrix mel = new Matrix(filterWeights.rows, m.columns);
		
		/* Performance optimization of ...

		mel = filterWeights.Multiply(m);
		for (int i = 0; i < mel.rows; i++) {
			for (int j = 0; j < mel.columns; j++) {
				mel.d[i, j] = (mel.d[i, j] < 1.0f ?
						        0 : (float)(10.0 * Math.Log10(mel.d[i, j])));
			}
		}*/
		
		unsafe {

			int mc = m.columns;
			int mr = m.rows;
			int melcolumns = mel.columns;
			int fwc = filterWeights.columns;
			int fwr = filterWeights.rows;
			int idx;
			int i;
			int k;
			int j;
			int kfwc;
			
			fixed (float* md = m.d, fwd = filterWeights.d, meld = mel.d) {
				for (i = 0; i < mc; i++) {
					for (k = 0; k < fwr; k++) {
						idx = k*melcolumns + i;
						kfwc = k*fwc;

						for (j = 0; j < mr; j++) {
							meld[idx] += fwd[kfwc + j] * md[j*mc + i];
						}

						meld[idx] = (meld[idx] < 1.0f ?
						        0 : (float)(10.0 * Math.Log10(meld[idx])));
					}
					
				}
			}
		}
		
		Matrix mfcc = dct.Multiply(mel);
		
		Dbg.WriteLine("mfcc Execution Time: " + t.Stop() + "ms");
		
		return mfcc;
	}
	
}
	
}
